##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
from string import join, strip
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import WorkflowCore
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from LinkPropertyManager import _getNewId, LinkPropertyManager, InitializeClassLinkPropertyManager
# Import permission names
import PropertyCorePermissions

def beforeCreate(id, prefix=None, REQUEST=None):
    if id is None or id == '' or id == 'None':
        id = _getNewId(prefix)
        if REQUEST is not None:
            REQUEST.set('id', id)
    return id

def afterCreate(self, id, obj, REQUEST=None):
    self._setObject(id, obj)

WorkflowMethod = WorkflowCore.WorkflowMethod

def InitializeClassPropertyContent(klass
                           , view_priviledge
                           , modify_priviledge
                           , property_extensible_schema__=None
                           , property_link_id_schema__=None
                           ):
    """
    Initialize Class with appropriate class security.
    """
    # Declarative security
    if not hasattr(klass, 'security'):
        klass.security = ClassSecurityInfo()
        klass.security.declareObjectProtected(view_priviledge)

    InitializeClassLinkPropertyManager(klass,modify_priviledge,property_extensible_schema__,property_link_id_schema__)

    klass.security.declareProtected(modify_priviledge, 'editMetadata')
    
    # Initialize the class, setting up security.
    InitializeClass(klass)

factory_type_information = (
    { 'id'                : 'PropertyContent'
      , 'meta_type'       : 'CMF PropertyContent'
      , 'description'     : """\
Property content object."""
      , 'icon'            : 'pcore_object_icon.gif'
      , 'product'         : 'CMFPropertyCore'
      , 'factory'         : 'manage_addPropertyContent'
      , 'immediate_view'  : 'pcore_edit_form'
      , 'actions'         :
      ( { 'name'          : 'View'
          , 'action'      : 'pcore_view'
          , 'permissions' : (PropertyCorePermissions.View,)
          }
      , { 'name'          : 'Edit'
          , 'action'      : 'pcore_edit_form'
          , 'permissions' : (PropertyCorePermissions.Modify,)
          }
      , { 'name'          : 'Metadata'
          , 'action'      : 'metadata_edit_form'
          , 'permissions' : (PropertyCorePermissions.Modify,)
          }
        )
      }
    ,)

def manage_addPropertyContent(self, id=None, title=None, parent=None, REQUEST=None):
    """
    Create an empty property content object.
    """
    id = beforeCreate(id, 'pcc', REQUEST)
    obj = PropertyContent(id, title, parent)
    afterCreate(self, id, obj, REQUEST)
    return id

class PropertyContent(LinkPropertyManager, PortalContent, DefaultDublinCoreImpl):
    """
    PropertyContent are property objects for the CMF.
    """
    meta_type='CMF PropertyContent'

    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(PropertyCorePermissions.View)
    
    #######################################
    # init
    def __init__(self, id, title=None, parent=None
                 , property_extensible_schema__=None
                 , property_link_id_schema__=None):
        LinkPropertyManager.__init__(self,id,title,parent
                                     , property_extensible_schema__
                                     , property_link_id_schema__)
        DefaultDublinCoreImpl.__init__(self)

    def manage_afterAdd(self, item, container):
        PortalContent.manage_afterAdd(self, item, container)

    def manage_beforeDelete(self, item, container):
        PortalContent.manage_beforeDelete(self, item, container)

    security.declareProtected(PropertyCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """
        text for indexing
        """
        from types import TupleType, ListType

        tokens = filter(lambda x,t=TupleType,l=ListType: (
            x and not (isinstance(x,t) or isinstance(x,l))), self.propertyValues())
        tokens = map(lambda x: strip(str(x)), tokens)
        return strip(join(tokens))

    manage_addProperty = WorkflowMethod(LinkPropertyManager.manage_addProperty)
    manage_delProperties = WorkflowMethod(LinkPropertyManager.manage_delProperties)
    manage_changeProperties = WorkflowMethod(LinkPropertyManager.manage_changeProperties)
    manage_changePropertyTypes = WorkflowMethod(LinkPropertyManager.manage_changePropertyTypes)
    manage_firstProperties = WorkflowMethod(LinkPropertyManager.manage_firstProperties)
    manage_lastProperties = WorkflowMethod(LinkPropertyManager.manage_lastProperties)

# Initialize the PropertyContent class, setting up security.
InitializeClassPropertyContent(PropertyContent
                               , PropertyCorePermissions.View
                               , PropertyCorePermissions.Modify
                               , 1
                               )
